= How to use Amrita2 with Ruby On Rails

== Summary

Amrita2 provides a Rails-Amrita2 bridge based on the work of of Jamis Buck.

  * http://one.textdrive.com/pipermail/rails/2005-February/003169.html
  * http://ruby.jamisbuck.org/amrita-view.zip

With this bridege installed, you can use Amrita2-stype XHTML template
instead of ERB template.

== Setup

Amrita2 archive is a Rails Plugin. So the only thing to do is
extracting Amrita2 archive at your_app/vendor/plugins.

Rails will load Amrita2 and amrita-rails bridge automatically by
evaluating amrita2/init.rb 

--- rubysrc ../init.rb

This initialization file orders Rails to treat template with extention
".a2html" as an Amrita2 template. If you put a template file named
"(action_name).a2html", it will loaded as (({Amrita2::Template})) and
controlers and helper methods will be used as Amrita2's Presentation
Object.

== 'hello world' tutorial

Set up a Rails Application
  
  $ rails amrita2_test

Extract Amrita2 archive as Rails plugin  

  $ cd amrita2_test/vendor/plugins
  $ tar zxvf amrita2_xxxxx.tar.gz

and create a application skelton with this command.

  $ cd amrita2_test
  $ ./script/generate controler hello 


Start the application

  $ cd amrita2_test
  $ ruby script/server

=== Hello World: Views 

Create 'app/views/hello/index.a2html' and 

--- xmlinline 
      # 
        <html>
          <body>
            <h1 id='title'>hello world</h1>
            <p id='body'>Amrita2 is a html template library for Ruby</p>
          </body>
        </html>


=== Hello World: Controlers 

Edit 'controllers/hello_controller.rb' like this

--- rubyinline 
      # 
        class HelloController < ApplicationController
          attr_reader :title, :body
          def index
            @title = 'Hello ROR with Amrita2'
            @body = 'Using Amrita2 with ROR is easy and fun!'
          end

Amrita2 use the Controler Object after calling the action method. So put a
method with same name as template's id attribute. Amrita2 will expand
the template with the returned value.

Now, after editing, see the 'http://localhost:3000/hello/' .

If everything is OK, the result will be

--- xmlinline 
      # 
        <html>
          <body>
            <h1>Hello ROR with Amrita2</h1>
            <p>Using Amrita2 with ROR is easy and fun!</p>
          </body>
        </html>

As you can see, Amrita2 uses contorollers as model data and insert the
value of instance variables of it to matching element of the template.

=== using model and make HTML tables

Now, let's use a Model object.

  ./script/generate model Language

Edit app/models/language.rb as below.

--- rubyinline 
      # 
        # app/models/language.rb
        class Language < Struct.new(:name, :creator)
          def self.find_all
            [
              self.new("Ruby", "matz" ),
              self.new("perl", "Larry Wall" ),
              self.new("python", "Guido van Rossum")
            ]
          end
        end

Edit controllers/hello_controller.rb

--- rubyinline 
      # 
        def languages
          @languages = Language.find_all
        end

Create views/hello/languages.a2html

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr id='languages'>
            <td id='name'></td><td id='creator'></td>
          </tr>
        </table>
        </body>
        </html>

access 'http://localhost:3000/languages', you will see this table.

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr>
            <td>Ruby</td><td>matz</td>
          </tr><tr>
            <td>perl</td><td>Larry Wall</td>
          </tr><tr>
            <td>python</td><td>Guido van Rossum</td>
          </tr>
        </table>
        </body>
        </html>

An Active-Record object can be used as a Presentation Object like this.


== Using partial ERB template

Add url to app/models/language.rb

--- rubyinline 
      # 
        class Language < Struct.new(:name, :creator, :url)
          def self.find_all
            [
              self.new("Ruby", "matz", "http://www.ruby-lang.org/"),
              self.new("perl", "Larry Wall", "http://www.perl.com/" ),
              self.new("python", "Guido van Rossum", "http://www.python.org/")
            ]
          end
        end

Edit views/hello/languages.a2html

--- xmlinline 
      # 
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr id='languages'>
            <td><![CDATA[
               <a href='<%= $_.url %>'><%= $_.name %></a>
            ]]></td>
            <td id='creator'></td>
          </tr>
        </table>

You can add any ERB sources in <![CDATA[...]]>. And you can use $_ which we call
context_value. Context_value is a value asigned at the element with id that 
contains ERB source.

In this case, (({<%= $_.url %>})) is  in the (({<tr id='languages'>..</tr>})) element,
so, (({$_})) is assinged a Language object.

So, the result is 

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr>
            <td><a href='http://www.ruby-lang.org/'>Ruby</a></td>
            <td>matz</td>
          </tr><tr>
            <td><a href='http://www.perl.com/'>perl</a></td>
            <td>Larry Wall</td>
          </tr><tr>
            <td><a href='http://www.python.org/'>python</a></td>
            <td>Guido van Rossum</td>
          </tr>
        </table>
        </body>
        </html>


== Using action_po method

If you like plain HTML template with out ERB or CDATA, then
add a PO method at helpers/hello_helper.rb

--- rubyinline 
      # 
        module HelloHelper
          include Amrita2
          def languages_po
            { 
              :languages => controller.languages.collect do |l|
                { 
                  :name => a(:href=>l.url) { l.name },
                  :creator => l.creator
                }
              end
            }
          end
        end

If the controler or helper Module have a method named '(action_name)_po',
Amrita2-Rails bridge will call this method befor template expansion.
And use the result of it as a Presentatio Object.

If the direct result of Models can not be used for template, 
you can modify or generate PO from the Model using PO method.


With this method, Rails-Amrita2 bridege generates the same result as
above with views/hello/languages.a2html

--- xmlinline 
      # 
        <html>
        <body>
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr id='languages'>
            <td><a id='name' /></td><td id='creator'></td>
          </tr>
        </table>
        </body>
        </html>

== using template setup method

If a method named '(method_name)_setup_template ' was defined in helper module,
Amrita2-Rails bridge will call this method for template compiling.

You can control detail of Amrita2 providing this method.

--- rubyinline 
      # 
        def languages_setup_template(template)
          tmpl = Amrita2::TemplateText.new(template)
          tmpl.use_erb(binding)
          tmpl.element_option[:languages] =  { :mv_attr=> { :class=>%w(odd even) } }
          tmpl
        end


With this method, you can use full feature of Amrita2::Template like
setting (({element_option})). This sample adds two (({class})) attribute values 
alternately  to (({<tr>})) element.

--- xmlinline 
      # 
        <table border='1'>
          <tr><th>name</th><th>author</th></tr>
          <tr class='odd'>
            <td><a href='http://www.ruby-lang.org/'>Ruby</a></td>
            <td>matz</td>
          </tr><tr class='even'>
            <td><a href='http://www.perl.com/'>perl</a></td>
            <td>Larry Wall</td>
          </tr><tr class='odd'>
            <td><a href='http://www.python.org/'>python</a></td>
            <td>Guido van Rossum</td>
          </tr>
        </table>


== Using scaffold_amrita generator

(under construction)

