require 'rubygems'
require 'amrita2/rd'
require 'syntax/convertors/html'

load File::dirname(File::dirname(__FILE__)) + "/lib/amrita2/version.rb"

APIDOC_ROOT="app"

class RDA
  include Amrita2
  include Amrita2::Runtime

  TMPLTEXT = <<-'END'
  <div>
    <amrita:erb id='rubysrc'>
      <table class='srcbox'>
      <tr><td align='center'><span class='srctitle'><%= $_[1] %></span></td></tr>
      <tr><td class='ruby'><%= rubysrc(*$_[1..2]) %></td></tr>
      </table>
    </amrita:erb>
    <amrita:erb id='rubyinline'>
      <table class='srcbox'>
        <tr><td class='ruby'><%= rubyinline($_[0]) %></td></tr>
      </table>
    </amrita:erb>
    <amrita:erb id='xmlsrc'>
      <table class='srcbox'>
      <tr><td align='center'><span class='srctitle'><%= $_[1] %></span></td></tr>
      <tr><td class='xml'><%= xmlsrc($_[1]) %></td></tr>
      </table>
    </amrita:erb>
    <amrita:erb id='xmlinline'>
      <table class='srcbox'>
        <tr><td class='xml'><%= xmlinline($_[0]) %></td></tr>
      </table>
    </amrita:erb>
    <amrita:erb id='amrita2_version'>
    <%= Amrita2::Version::const_get($_||'STRING') %>
    </amrita:erb>
    <amrita:erb id='API'>
    <%= link_to_api_doc(*$_) %>
    </amrita:erb>
  </div>
  END

  RubyConvertor = Syntax::Convertors::HTML.for_syntax "ruby"
  XMLConvertor = Syntax::Convertors::HTML.for_syntax "xml"

  def rubysrc(name, title=name)
    src = File::open(name).read 
    if /\A(.*)__END__(.*)\Z/m =~ src
      src, result = $1, $2
      RubyConvertor.convert(src) +
          "output<span class='xml'>" + XMLConvertor.convert(result.strip) + "</span>"
    else
      RubyConvertor.convert(src)
    end
  end

  def rubyinline(src)
    src = unescape(src)
    RubyConvertor.convert(src)
  end

  def xmlsrc(name, title=name)
    src = File::open(name).read 
    XMLConvertor.convert(src.strip)
  end

  def xmlinline(src)
    src = unescape(src)
    XMLConvertor.convert(src)
  end

  def link_to_api_doc(para, text=nil)
    clss = para.split('::')
    clss[-1], meth = *clss[-1].split('#')
    href = "#{APIDOC_ROOT}/classes/#{clss.join('/')}.html"

    unless text
      text = clss.join('::') 
      text += "##{meth}" if meth
    end
    "<a href='#{href}'><code class='api'>#{text}</code></a>"
  end

  def unescape(src)
    src = src.gsub(%r[<p>#\s*</p>], "")
    src.gsub!(%r[\A<pre>], "")
    src.gsub!(%r[</pre>\Z], "")
    src.gsub!("&lt;", "<")
    src.gsub!("&gt;", ">")
    src
  end

  def exttmpl
    t = Amrita2::TemplateText.new TMPLTEXT
    #t.debug_source=STDOUT
    t.use_erb(binding)
    parts = [
      :rubysrc,
      :rubyinline,
      :xmlsrc,
      :xmlinline,
      :amrita2_version,
      :API
    ]
    t.define_parts(*parts)

    t
  end

  def process(f)
    to = File::basename(f, '.rd') + '.html'
    lang = ((File::basename(f, '.rd') =~ /J$/)?"EUC-JP":nil)
    Amrita2::RDAProcessor.compile_rd(f, to, 
      :css=>"default.css", :exttmpl=>exttmpl, :lang=>lang)
  end
end

dir = ARGV.shift

rda = RDA.new
Dir::chdir(dir) do
  Dir::glob("*.rd") do |f|
    puts f
    rda.process(f)
  end
end

