#!/usr/bin/env ruby

require "amrita2/template"
include Amrita2

$exclude = %w( CVS index.html)
$tmpl = TemplateText.new <<END
<html>
  <head>
    <title id="title"></title>
  </head>
  <body>
    <h1 id="title"></h1>
    <hr />
    <table>
      <tr>  <th>Type</th><th>Name</th><th>Size</th> </tr>
      <tr id="filelist">
        <td id="filetype" /><td id="name" /><td id="size" align="right" />
      </tr>
    </table>
    <hr />
    <a href="../index.html">parent directory</a>
  </body>
</html>
END

class File
  class Stat
    attr_accessor :name
    
    def filetype
      case ftype
      when "directory"
        "[DIR]"
      when "file"
        "[   ]"
      else
        STDERR.puts "unknown file type (#{ftype}) #{name}"
      end
    end
  end
end

def make_filelist(dir=nil)
  #puts dir
  d = Dir::pwd
  Dir::chdir dir if dir
  l = Dir['*'].sort.collect do |f|
    next if $exclude.member? f
    s = File::stat(f)
    path = f
    path += "/" if s.ftype == "directory"
    s.name = e(:a, :href=>path) { f } 
    make_filelist(f) if s.ftype == "directory"
    s
  end

  data = {
    :title => dir,
    :filelist=>l
  }
  unless FileTest::exists?("index.html")
    File::open("index.html", "w") do |f|
      $tmpl.expand(f, data)
    end
  end
ensure
  Dir::chdir d
end

make_filelist ARGV.shift

