require "amrita2/template"
include Amrita2


TEMPLATE_TEXT = <<END
<html>
   <body>
     <h1><a id='a1'>replacing attributes</a></h1>
     <p>See <a id='a2' href='http://www.ruby-lang.org/'>Ruby homepage</a> for detail</p>
     <a id='rubytalk' href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">[ruby-talk:$1]</a>
     <a id='raa' href="http://raa.ruby-lang.org/list.rhtml?name=$1">[$2]</a>
   </body>
</html>
END

# You need to use template spec for replacing attribute values
# There are three ways for it.
tmpl = TemplateText.new(TEMPLATE_TEXT) do
  Amrita2::define_template_spec do 
    dynamic_element(:a1)
    dynamic_element(:a2, :use_original_element=>true)
    dynamic_element(:rubytalk, :use_args=>true)
    dynamic_element(:raa, :use_args=>true)
  end
end
class PO
  def a1(m)
    # replace_attr: optional Hash will be merged to attributes
    m.a1('replaced', :href=>'http://amrita2.rubyforge.org/')
  end

  def a2(m)
    # use_original_element: REXML::Element object for the spec
    # will be passed. Edit it and return it or make a new
    # Element object and return it.
    m.a2 do |e|
      e.text += "(Japanese)"
      e.attributes['href'] += 'ja/'
      e
    end
  end

  def rubytalk(m)
    # use_args: the value passed will be replaced $1, $2.....
    m.rubytalk(132204)
  end

  def raa(m)
    m.raa('amrita', 'amrita: HTML template library')
  end
end

tmpl.expand(STDOUT, PO.new)

__END__
# The result is
<html>
   <body>
     <h1><a href='http://amrita2.rubyforge.org/' id='a1'>replaced</a></h1>
     <p>See <a href='http://www.ruby-lang.org/ja/' id='a2'>Ruby homepage(Japanese)</a> for detail</p>
     <a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/132204'>[ruby-talk:132204]</a>
     <a href='http://raa.ruby-lang.org/list.rhtml?name=amrita'>[amrita: HTML template library]</a>
   </body>
</html>
